@@ -209,7 +209,7 @@ boot_settings_fill_type_list (BootSettingsGui *gui)
 
 	root  = gst_xml_doc_get_root (tool->config);
 	part  = gst_xml_element_find_first (root, "partitions");
-	
+
 	for (node = gst_xml_element_find_first (part, "partition");
 	     node; node = gst_xml_element_find_next (node, "partition"))
 	{
@@ -309,6 +309,9 @@ boot_settings_gui_setup (BootSettingsGui *gui, GtkWidget *top)
 	{
 		settings_dev_list (GTK_COMBO_BOX (gui->device), image->type);
 
+		if (!image->new)
+			boot_settings_set_type (gui, type_to_label (image->type));
+
 		if (image->image)
 			gst_ui_entry_set_text (GTK_ENTRY (GTK_BIN (gui->device)->child), image->image);
 	}
@@ -364,7 +367,11 @@ boot_settings_gui_save (BootSettingsGui *gui, gboolean check)
 	if (image->append)   g_free (image->append);
 	if (image->password) g_free (image->password);
 
-	image->type = label_to_type (boot_settings_get_type (gui));
+	/* only check if the image is new, if it isn't
+	   then there's no possibility to change it */
+	if (image->new)
+		image->type = label_to_type (boot_settings_get_type (gui));
+
 	image->label = g_strdup (gtk_entry_get_text (gui->name));
 
 	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (gui->use_password)))
