@@ -32,6 +32,7 @@
 #include "gst-auth.h"
 #include "gst-xml.h"
 #include "gst-marshal.h"
+#include "gst-hig-dialog.h"
 
 #ifdef GST_HAVE_ARCHIVER
 #  include <bonobo.h>
@@ -54,6 +55,7 @@
 #define BUFFER_SIZE 2048
 #define GST_DEBUG 1
 #define GST_TOOL_EOR "<!-- GST: end of request -->"
+#define REDRAW_NCHARS 50
 
 /* Define this if you want the frontend to run the backend under strace */
 /*#define GST_DEBUG_STRACE_BACKEND*/
@@ -70,6 +72,10 @@ GdkPixbuf *gentoo;
 GdkPixbuf *pld;
 GdkPixbuf *openna;
 GdkPixbuf *fedora;
+GdkPixbuf *conectiva;
+GdkPixbuf *black_panther;
+GdkPixbuf *vine;
+GdkPixbuf *specifix;
 
 enum {
 	BOGUS,
@@ -87,11 +93,12 @@ enum {
 	PLATFORM_LIST_COL_LAST
 };
 
-static GtkObjectClass *parent_class;
+static GObjectClass *parent_class;
 static gint gsttool_signals [LAST_SIGNAL] = { 0 };
 
 static void     gst_tool_idle_run_directives_remove (GstTool *tool);
 static void     gst_tool_idle_run_directives_add    (GstTool *tool);
+static void     gst_tool_directive_free             (GstDirectiveEntry*);
 
 static gboolean platform_set_current_cb   (GstTool *tool, GstReportLine *rline, gpointer data);
 static gboolean platform_unsupported_cb   (GstTool *tool, GstReportLine *rline, gpointer data);
@@ -170,7 +177,6 @@ report_finished_cb (GstTool *tool, GstReportLine *rline, gpointer data)
 	tool->report_finished = TRUE;
 
 	gtk_widget_hide (tool->report_window);
-	gtk_main_quit ();
 	
 	return TRUE;
 }
@@ -310,21 +316,19 @@ report_dispatch (GstTool *tool)
 }
 
 static void
-report_progress_tick (gpointer data, gint fd, GdkInputCondition cond)
+report_progress_do (GstTool *tool)
 {
-	GstTool *tool;
 	GstReportLine *rline;
 	gchar *buffer;
 	gint i = 0;
 
-	tool = GST_TOOL (data);
 	if (tool->input_block)
 		return;
 
 	if (!tool->line)
 		tool->line = g_string_new ("");
 
-	while ((buffer = gst_tool_read_from_backend (tool))) {
+	while ((buffer = gst_tool_read_line_from_backend (tool))) {
 		for (i = 0; (i < strlen (buffer)); i++) {
 			gchar c = buffer [i];
 
@@ -352,6 +356,8 @@ report_progress_tick (gpointer data, gint fd, GdkInputCondition cond)
 				g_string_append_c (tool->line, buffer [i]);
 			}
 		}
+
+		g_free (buffer);
 	}
 
 full_break:
@@ -371,6 +377,8 @@ full_break:
 		gdk_input_remove (tool->input_id);
 		tool->input_id = 0;
 	}
+
+	g_free (buffer);
 }
 
 static gboolean
@@ -382,38 +390,18 @@ report_window_close_cb (GtkWidget *window, GdkEventAny *ev, gpointer data)
 }
 
 static void
-report_progress (GstTool *tool, const gchar *label)
+report_progress (GstTool *tool)
 {
-	tool->timeout_done = FALSE;
-	tool->report_list_visible = FALSE;
 	tool->report_finished = FALSE;
 	tool->report_dispatch_pending = FALSE;
 
 	gst_tool_clear_supported_platforms (tool);
 	report_clear_lines (tool);
 
-	if (label) {
-		gtk_label_set_text (GTK_LABEL (tool->report_label), label);
-		gtk_widget_show_all (tool->report_window);
-		gtk_widget_show_now (tool->report_window);
+	report_progress_do (tool);
 
-		/* This ensures the report_window will be hidden on time.
-		 * I'ts OK for long-lived directives, and short lived shouldn't
-		 * be using the report window anyways. */
-		while (gtk_events_pending ()) {
-			usleep (1);
-			gtk_main_iteration ();
-		}
-	}
-
-	tool->input_id = gtk_input_add_full (tool->backend_master_fd, GDK_INPUT_READ,
-					     report_progress_tick, NULL, tool, NULL);
-	tool->input_block = FALSE;
-
-	gtk_main ();
-
-	if (tool->input_id)
-		gtk_input_remove (tool->input_id);
+	while (gtk_events_pending ())
+		gtk_main_iteration ();
 }
 
 static GSList *
@@ -454,6 +442,10 @@ gst_tool_create_distro_images (void)
 	pld = gdk_pixbuf_new_from_file (PIXMAPS_DIR "/pld.png", NULL);
 	openna = gdk_pixbuf_new_from_file (PIXMAPS_DIR "/openna.png", NULL);
 	fedora = gdk_pixbuf_new_from_file (PIXMAPS_DIR "/fedora.png", NULL);
+	conectiva = gdk_pixbuf_new_from_file (PIXMAPS_DIR "/conectiva.png", NULL);
+	black_panther = gdk_pixbuf_new_from_file (PIXMAPS_DIR "/black_panther.png", NULL);
+	vine = gdk_pixbuf_new_from_file (PIXMAPS_DIR "/vine.png", NULL);
+	specifix = gdk_pixbuf_new_from_file (PIXMAPS_DIR "/specifix.png", NULL);
 }
 
 static void
@@ -523,7 +515,7 @@ gst_tool_process_startup (GstTool *tool)
 	/* let's process those startup reports. */
 	tool->report_hook_type = GST_REPORT_HOOK_LOAD;
 
-	report_progress (tool, NULL);
+	report_progress (tool);
 
 	/* stable version users should pass here only once,
 	   but if there's an inconsistency the dialog will repeat */
@@ -553,7 +545,7 @@ gst_tool_idle_run_directives (gpointer data)
 		entry = tool->directive_queue->data;
 		tool->directive_queue = g_slist_remove (tool->directive_queue, entry);
 		entry->callback (entry);
-		g_free (entry);
+		gst_tool_directive_free (entry);
 	}
 
 	tool->directive_queue_idle_id = 0;
@@ -607,8 +599,9 @@ gst_tool_kill_tool_cb (GstDirectiveEntry *entry)
 	GstTool *tool = entry->tool;
 
 	gst_tool_kill_backend_cb (entry);
-	
-	g_signal_emit_by_name (G_OBJECT (tool), "destroy");
+
+	g_object_unref (G_OBJECT (tool));
+	exit (0);
 }
 
 /* kills the whole tool */
@@ -653,23 +646,22 @@ gst_tool_end_of_request (GString *string)
 static xmlDoc *
 gst_tool_read_xml_from_backend (GstTool *tool)
 {
-	gchar *buffer;
-	int t;
-	xmlDoc *xml;
+	xmlDoc *xml = NULL;
+	gchar *buffer = NULL;
 
 	if (!tool->xml_document)
 		return NULL;
 
-	while (! gst_tool_end_of_request (tool->xml_document)) {
-		buffer = gst_tool_read_from_backend (tool);
+	do {
+		g_free (buffer);
+		buffer = gst_tool_read_line_from_backend (tool);
 		g_string_append (tool->xml_document, buffer);
-	}
+	} while (g_strrstr (buffer, GST_TOOL_EOR "\r\n") == NULL);
 
-	if (tool->xml_document->str[0] == '<') {
+	g_free (buffer);
+
+	if (tool->xml_document->str[0] == '<')
 		xml = xmlParseDoc (tool->xml_document->str);
-	} else {
-		xml = NULL;
-	}
 
 	g_string_free (tool->xml_document, TRUE);
 	tool->xml_document = NULL;
@@ -711,6 +703,15 @@ gst_tool_queue_directive (GstTool *tool, GstDirectiveFunc callback, gpointer dat
 	gst_tool_idle_run_directives_add (tool);
 }
 
+static void
+gst_tool_directive_free (GstDirectiveEntry *entry)
+{
+	g_free (entry->report_sign);
+	xmlFreeDoc (entry->in_xml);
+
+	g_free (entry);
+}
+
 /* As specified, escapes :: to \:: and \ to \\ and joins the strings. */
 static GString *
 gst_tool_join_directive (const gchar *directive, va_list ap)
@@ -783,7 +784,7 @@ gst_tool_run_get_directive_va (GstTool *tool, const gchar *report_sign, const gc
 		tool->xml_document = g_string_new ("");
 
 	if (location_id == NULL)
-		report_progress (tool, report_sign? _(report_sign): NULL);
+		report_progress (tool);
 
 	xml = gst_tool_read_xml_from_backend(tool);
 
@@ -819,32 +820,28 @@ gst_tool_run_set_directive_va (GstTool *tool, xmlDoc *xml,
 
 	/* don't actually run if we are just pretending */
 	if (tool->root_access == ROOT_ACCESS_SIMULATED) {
-		g_warning (_("Skipping %s directive..."), directive);
+		g_warning ("Skipping %s directive...", directive);
 		tool->directive_running = FALSE;
 		return NULL;
 	}
 
 	gst_tool_send_directive (tool, directive, ap);
 
-	if (xml) {
+	if (xml)
 		gst_tool_write_xml_to_backend (tool, xml);
-		gst_tool_write_to_backend (tool, "\n");
-		gst_tool_write_to_backend (tool, GST_TOOL_EOR);
-		gst_tool_write_to_backend (tool, "\n");
-	}
 
+	/* This is tipicaly to just read the end of request string,
+	   but a set directive may return some XML too. */
+	xml_out = gst_tool_read_xml_from_backend (tool);
 
 	/* FIXME: Instead of doing the following, we should pass around a value describing
 	 * tool I/O mode (as opposed to a string to report_progress ()). */
 	tool->report_hook_type = GST_REPORT_HOOK_SAVE;
 
 	if (location_id == NULL)
-		report_progress (tool, report_sign? _(report_sign): NULL);
+		report_progress (tool);
 
-
-	/* This is tipicaly to just read the end of request string,
-	   but a set directive may return some XML too. */
-	xml_out = gst_tool_read_xml_from_backend (tool);
+	gst_tool_read_junk_from_backend (tool, GST_TOOL_EOR "\r\n");
 
 	tool->directive_running = FALSE;
 	gst_tool_idle_run_directives_add (tool);
@@ -923,10 +920,9 @@ gst_tool_load (GstTool *tool)
 #endif	
 	}
 
+	if (tool->config)
+		g_signal_emit (G_OBJECT (tool), gsttool_signals[FILL_GUI], 0);
 
-	if (tool->config){
-		g_signal_emit (GTK_OBJECT (tool), gsttool_signals[FILL_GUI], 0);
-	}
 	return tool->config != NULL;
 }
 
@@ -946,11 +942,11 @@ gst_tool_save (GstTool *tool, gboolean restore)
 
 	gst_dialog_freeze_visible (tool->main_dialog);
 
-	g_signal_emit (GTK_OBJECT (tool), gsttool_signals[FILL_XML], 0);
+	g_signal_emit (G_OBJECT (tool), gsttool_signals[FILL_XML], 0);
 #ifdef GST_DEBUG
 	/* don't actually save if we are just pretending */
 	if (tool->root_access == ROOT_ACCESS_SIMULATED) {
-		g_warning (_("Skipping actual save..."));
+		g_warning ("Skipping actual save...");
 		return TRUE;
 	}
 #endif
@@ -1009,23 +1005,19 @@ gst_tool_save_cb (GtkWidget *w, GstTool *tool)
 	gst_tool_save (tool, FALSE);
 }
 
-void
-gst_tool_restore_cb (GtkWidget *w, GstTool *tool)
-{
-	gst_tool_save (tool, TRUE);
-}
-
 void
 gst_tool_load_try (GstTool *tool)
 {
 	if (!gst_tool_load (tool)) {
 		GtkWidget *d;
 
-		d = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR,
-					    GTK_BUTTONS_CLOSE,
-					    _("There was an error running the backend script,\n"
-					      "and the configuration could not be loaded."));
-
+		d = gst_hig_dialog_new (GTK_WINDOW (tool->main_dialog),
+					GTK_DIALOG_MODAL,
+					GST_HIG_MESSAGE_ERROR,
+					_("The configuration could not be loaded"),
+					_("There was an error running the backend script"),
+					GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
+					NULL);
 		gtk_dialog_run (GTK_DIALOG (d));
 		gtk_widget_destroy (d);
 		
@@ -1073,28 +1065,23 @@ gst_tool_main_with_hidden_dialog (GstTool *tool, gboolean no_main_loop)
 }
 
 static void
-gst_tool_destroy (GtkObject *object)
+gst_tool_finalize (GObject *object)
 {
-	GstTool *tool;
-
-	tool = GST_TOOL (object);
-	
-	parent_class->destroy (object);
+	parent_class->finalize (object);
 	gtk_main_quit ();
 }
 
 static void
-gst_tool_class_init (GstToolClass *klass)
+gst_tool_class_init (GstToolClass *class)
 {	
-	GtkObjectClass *object_class;
+	GObjectClass *gobject_class;
 
-	object_class = (GtkObjectClass *)klass;
-
-	parent_class = g_type_class_peek_parent (klass);
+	gobject_class = G_OBJECT_CLASS (class);
+	parent_class = g_type_class_peek_parent (class);
 
 	gsttool_signals[FILL_GUI] = 
 		g_signal_new ("fill_gui",
-			      G_OBJECT_CLASS_TYPE (object_class),
+			      G_OBJECT_CLASS_TYPE (gobject_class),
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (GstToolClass, fill_gui),
 			      NULL, NULL,
@@ -1102,7 +1089,7 @@ gst_tool_class_init (GstToolClass *klass)
 			      G_TYPE_NONE, 0);
 	gsttool_signals[FILL_XML] = 
 		g_signal_new ("fill_xml",
-			      G_OBJECT_CLASS_TYPE (object_class),
+			      G_OBJECT_CLASS_TYPE (gobject_class),
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (GstToolClass, fill_xml),
 			      NULL, NULL,
@@ -1110,14 +1097,14 @@ gst_tool_class_init (GstToolClass *klass)
 			      G_TYPE_NONE, 0);
 	gsttool_signals[CLOSE] = 
 		g_signal_new ("close",
-			      G_OBJECT_CLASS_TYPE (object_class),
+			      G_OBJECT_CLASS_TYPE (gobject_class),
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (GstToolClass, close),
 			      NULL, NULL,
 			      gst_marshal_VOID__VOID,
 			      G_TYPE_NONE, 0);
 
-	object_class->destroy = gst_tool_destroy;
+	gobject_class->finalize = gst_tool_finalize;
 }
 
 static void
@@ -1245,13 +1232,11 @@ gst_tool_type_init (GstTool *tool)
 
 	tool->glade_common_path  = g_strdup_printf ("%s/common.glade", INTERFACES_DIR);
 
-	tool->etspecs_common_path = g_strdup (ETSPECS_DIR);
-
 	tool->report_gui = xml  = gst_tool_load_glade_common (tool, "report_window");
 
 	tool->report_window     = glade_xml_get_widget (xml, "report_window");
-	g_signal_connect (GTK_OBJECT (tool->report_window), "delete_event",
-			    G_CALLBACK (report_window_close_cb), tool);
+	g_signal_connect (G_OBJECT (tool->report_window), "delete_event",
+			  G_CALLBACK (report_window_close_cb), tool);
 
 	tool->report_label      = glade_xml_get_widget (xml, "report_label");
 
@@ -1286,7 +1271,7 @@ gst_tool_get_type (void)
 			(GInstanceInitFunc) gst_tool_type_init
 		};
 		
-		gsttool_type = g_type_register_static (GTK_TYPE_OBJECT, "GstTool", &gsttool_info, 0);
+		gsttool_type = g_type_register_static (G_TYPE_OBJECT, "GstTool", &gsttool_info, 0);
 	}
 
 	return gsttool_type;
@@ -1337,10 +1322,6 @@ gst_tool_construct (GstTool *tool, const char *name, const char *title)
 			  "apply",
 			  G_CALLBACK (gst_tool_save_cb),
 			  tool);
-	g_signal_connect (G_OBJECT (tool->main_dialog),
-			  "restore",
-			  G_CALLBACK (gst_tool_restore_cb),
-			  tool);
 
 	tool->report_hook_list = NULL;
 	memset (&tool->report_hook_defaults, 0, sizeof (GstReportHook *) * GST_MAJOR_MAX);
@@ -1362,9 +1343,10 @@ gst_tool_construct (GstTool *tool, const char *name, const char *title)
 	hosts = gconf_client_get_string (client, remote_hosts_key, &error);
 
 	tool->remote_config = ((do_remote_conf) && (hosts != NULL) && (strcmp (SSH_PATH, "") != 0));
-	if (hosts != NULL) 
+	if (tool->remote_config && hosts)
 		tool->remote_hosts = g_strsplit (hosts, ",", -1);
 
+	g_free (hosts);
 	g_free (remote_config_key);
 	g_free (remote_hosts_key);
 }
@@ -1400,8 +1382,7 @@ gst_tool_set_close_func (GstTool *tool, GstCloseFunc close_cb, gpointer data)
 	g_return_if_fail (GST_IS_TOOL (tool));
 
 	if (close_cb)
-		g_signal_connect (GTK_OBJECT (tool), "close", G_CALLBACK (close_cb), data);
-
+		g_signal_connect (G_OBJECT (tool), "close", G_CALLBACK (close_cb), data);
 }
 
 void
@@ -1507,14 +1488,13 @@ try_show_usage_warning (void)
 	gchar *version;
 	GConfClient *client;
 	GError *error = NULL;
-	gchar *warning_title = g_strdup_printf (_("<span weight=\"bold\" size=\"larger\">"
-						  "Welcome to the %s prerelease of the GNOME System Tools"
-						  "</span>"), VERSION);
-	gchar *warning = _("This is still a work in progress, and so it may have serious bugs. "
-			   "Due to the nature of these tools, bugs may render your computer "
-			   "<span weight=\"bold\">practically useless</span>, costing time, effort and sanity points. "
-			   "You have been warned. Thank you for trying out this prerelease of "
-			   "the GNOME System Tools!");
+	gchar *title         = g_strdup_printf (_("Welcome to the %s prerelease of the GNOME System Tools"), VERSION);
+	gchar *warning_title = g_strdup_printf ("<span weight=\"bold\" size=\"larger\">%s</span>", title);
+	gchar *warning       = _("This is still a work in progress, and so it may have serious bugs. "
+				 "Due to the nature of these tools, bugs may render your computer "
+				 "<span weight=\"bold\">practically useless</span>, costing time, effort and sanity points. "
+				 "You have been warned. Thank you for trying out this prerelease of "
+				 "the GNOME System Tools!");
 
 	client = gconf_client_get_default ();
 
@@ -1583,6 +1563,7 @@ try_show_usage_warning (void)
 	}
 
 	g_free (warning_title);
+	g_free (title);
 	g_free (key);
 }
 
@@ -1624,40 +1605,113 @@ gst_init (const gchar *app_name, int argc, char *argv [], const poptOption optio
  	program = gnome_program_init (app_name, VERSION,
 				      LIBGNOMEUI_MODULE, argc, argv,
 				      GNOME_PARAM_POPT_TABLE, options,
+				      GNOME_PARAM_APP_DATADIR, DATADIR,
 				      GNOME_PARAM_HUMAN_READABLE_NAME,
 				      _("GNOME System Tools"),
 				      NULL);
-	try_show_usage_warning ();
+	/* try_show_usage_warning (); */
 }
 
-gchar*
-gst_tool_read_from_backend (GstTool *tool)
+static gboolean
+is_string_complete (gchar *str, GSList *list)
 {
-	gchar *buffer;
-	GString *str = g_string_new ("");
-	gchar *ptr;
-	gboolean may_exit = FALSE;
+	GSList *elem;
 	
-	buffer = (gchar*) g_malloc0 (sizeof (gchar) * BUFFER_SIZE);
+	if (strlen (str) == 0)
+		return FALSE;
 
-	do {
-		usleep (32000);
+	for (elem = list; elem; elem = g_slist_next (elem))
+		if (g_strrstr (str, elem->data) != NULL)
+			return TRUE;
+
+	return FALSE;
+}
+
+static gchar*
+gst_tool_read_from_backend_va (GstTool *tool, gchar *needle, va_list ap)
+{
+	GString *str = g_string_new ("");
+	gboolean may_exit = FALSE;
+	gint i = 0;
+	gchar c, *ptr, *arg;
+	GSList *list = NULL;
+
+	list = g_slist_prepend (list, needle);
+
+	while ((arg = va_arg (ap, char*)) != NULL)
+		list = g_slist_prepend (list, arg);
+	va_end (ap);
+
+	while (!is_string_complete (str->str, list)) {
+		c = fgetc (tool->backend_stream);
+		i++;
 		
-		while (fgets (buffer, BUFFER_SIZE, tool->backend_stream) != NULL) {
-			g_string_append (str, buffer);
-			may_exit = TRUE;
+		if (*str->str)
+			g_string_append_c (str, c);
+		else {
+			/* the string is still empty, read with O_NONBLOCK until
+			 *  it gets a char, this is done for not blocking the UI
+			 */
+			if (c != EOF) {
+				g_string_append_c (str, c);
+				fcntl (tool->backend_master_fd, F_SETFL, 0);
+			}
+			
+			usleep (500);
 		}
-		
-		while (gtk_events_pending ())
-			gtk_main_iteration ();
-	} while (!may_exit);
 
+		/* ugly hack for redrawing UI without too much overload */
+		if (i == REDRAW_NCHARS) {
+			while (gtk_events_pending ())
+				gtk_main_iteration ();
+			i = 0;
+		}
+	}
+
+	fcntl (tool->backend_master_fd, F_SETFL, O_NONBLOCK);
 	ptr = str->str;
 	g_string_free (str, FALSE);
 
 	return ptr;
 }
 
+gchar*
+gst_tool_read_from_backend (GstTool *tool, gchar *needle, ...)
+{
+	va_list ap;
+
+	va_start (ap, needle);
+	return gst_tool_read_from_backend_va (tool, needle, ap);
+}
+
+gchar*
+gst_tool_read_line_from_backend (GstTool *tool)
+{
+	gchar line[1000];
+
+	fcntl (tool->backend_master_fd, F_SETFL, 0);
+	fgets (line, 1000, tool->backend_stream);
+	fcntl (tool->backend_master_fd, F_SETFL, O_NONBLOCK);
+
+	while (gtk_events_pending ())
+		gtk_main_iteration ();
+
+	return g_strdup (line);
+}
+
+void
+gst_tool_read_junk_from_backend (GstTool *tool, gchar *needle)
+{
+	gchar *buffer = NULL;
+
+	do {
+		g_free (buffer);
+		buffer = gst_tool_read_line_from_backend (tool);
+	} while (g_strrstr (buffer, needle) == NULL);
+
+	g_free (buffer);
+}
+
 void
 gst_tool_write_xml_to_backend (GstTool *tool, xmlDoc *doc)
 {
@@ -1669,16 +1723,12 @@ gst_tool_write_xml_to_backend (GstTool *tool, xmlDoc *doc)
 	string = (gchar *) xml;
 	
 	gst_tool_write_to_backend (tool, string);
-
 	xmlFree (xml);
-
-	usleep (32000);
 }
 
 void
 gst_tool_write_to_backend (GstTool *tool, gchar *string)
 {
-	gint ntotal = 0;
 	gint nread = 0;
 	int ret;
 	gchar *p;
@@ -1691,9 +1741,42 @@ gst_tool_write_to_backend (GstTool *tool, gchar *string)
 
 		if (ret != EOF)
 			nread++;
+
+		/* ugly hack for redrawing UI */
+		if (nread % REDRAW_NCHARS == 0)
+			while (gtk_events_pending ())
+				gtk_main_iteration ();
 	} while (nread < strlen (string));
 
-	fcntl (tool->backend_master_fd, F_SETFL, O_NONBLOCK);
-	
 	while (fflush (tool->backend_stream) != 0);
+
+	fcntl (tool->backend_master_fd, F_SETFL, O_NONBLOCK);
+}
+
+void
+gst_tool_show_help (GstTool *tool, gchar *section)
+{
+	GError    *error = NULL;
+	gchar     *help_file;
+
+	help_file = g_strdup_printf ("%s-admin", tool->name);
+	gnome_help_display (help_file, section, &error);
+
+	if (error) {
+		GtkWidget *dialog;
+
+		dialog = gst_hig_dialog_new (GTK_WINDOW (tool->main_dialog),
+					     GTK_DIALOG_MODAL,
+					     GTK_MESSAGE_ERROR,
+					     _("Could not display help"),
+					     error->message,
+					     GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
+					     NULL);
+
+		gtk_dialog_run (GTK_DIALOG (dialog));
+		gtk_widget_destroy (dialog);
+		g_error_free (error);
+	}
+
+	g_free (help_file);
 }
