@@ -22,7 +22,6 @@
 #ifndef GST_TOOL_H
 #define GST_TOOL_H
 
-#include <gtk/gtkobject.h>
 #include <glade/glade.h>
 #include <popt.h>
 
@@ -45,7 +44,7 @@ typedef enum {
 } RootAccess;
 
 struct _GstTool {
-	GtkObject object;
+	GObject object;
 	char *name;
 
 	RootAccess root_access;
@@ -114,7 +113,7 @@ struct _GstTool {
 };
 
 struct _GstToolClass {
-	GtkObjectClass parent_class;
+	GObjectClass parent_class;
 
 	void (*fill_gui) (GstTool *xt);
 	void (*fill_xml) (GstTool *xt);
@@ -178,6 +177,7 @@ gchar*       gst_tool_read_line_from_backend (GstTool*);
 void         gst_tool_read_junk_from_backend (GstTool*, gchar*);
 void         gst_tool_write_to_backend (GstTool*, gchar*);
 void         gst_tool_write_xml_to_backend (GstTool*, xmlDoc*);
+void         gst_tool_show_help (GstTool*, gchar*);
 
 #define __full_tool_name(parameter) "Gst" #parameter "Tool"
 
