@@ -119,6 +119,7 @@ typedef struct {
 
 	/* Wavelan */
 	gchar *essid;
+	gchar *key;
 
 	/* PPP */
 	gchar *phone_number;
@@ -183,6 +184,8 @@ void connection_set_bcast_and_network (GstConnection *cxn);
 gboolean connection_poll_stat (GstTool*);
 gchar*   connection_autodetect_modem (void);
 gchar*   connection_description_from_type (GstConnectionType);
+void     connection_fill_ip_menu (GtkWidget*);
+void     connection_update_ip_config (GstConnection*);
 void     connection_enable (GstConnection*, gboolean);
 void     connection_apply_and_enable (GstTool*, GstConnection*);
 
